#include "GlobalConfig.h"
#include <QSettings>
#include <QCoreApplication>

#include "spdlog/spdlog.h"

#include "GlobalVariable.h"


GlobalVariable::GlobalVariable()
{
    ThreadSleepMS = 300;

    ActFaceIdentify.clear();
    ActPersonCount.clear();
    ActContraband.clear();
    ActPlayPhone.clear();
}

std::string GlobalVariable::getActionName(const std::string& actionID) 
{
    std::string actionName;
    mutexRW.lockForRead();
    auto it = m_mapAction.find(actionID);
    if(it != m_mapAction.end()) {
        actionName = it->second;
    }
    mutexRW.unlock();
    return actionName;
}

/* 设置算法名称和算法ID对应关系，没有算法不会添加 */
void GlobalVariable::setActionName(const std::map<std::string, std::string>& mapAction)
{
    mutexRW.lockForWrite();
    for(const auto& it : m_mapAction)
    {
        auto it0 = mapAction.find(it.first);
        if(it0 != mapAction.end())
        {
            m_mapAction[it.second] = it0->second;
        }
    }
    mutexRW.unlock();
}

/* 通过不带后缀的算法ID获取完整的算法ID，后缀通过“-”连接 */
std::string GlobalVariable::getFullActionID(const std::string& actionID)
{
    std::string fullActionID;
    for(const auto& it : m_mapAction)
    {
        /* 去掉“-”以及后面的字符串 */
        auto tmpStr = it.first;
        auto pos = tmpStr.find("-");
        if(pos != std::string::npos)
        {
            tmpStr = tmpStr.substr(0, pos);
        }
        if(tmpStr == actionID)
        {
            fullActionID = it.first;
            break;
        }
    }
    return fullActionID;
}

/* ====================================================================================
 * **************************    GlobalConfig成员函数    ******************************
 * ====================================================================================*/



GlobalConfig::GlobalConfig()
{
    
}


/* 初始化服务 */
void GlobalConfig::initService()
{
    /* 读取基础配置文件 */
    QString strBaseConfig = QCoreApplication::applicationDirPath() + "/config/BaseConfig.ini";
    if(!GConfig.readBaseConfig(strBaseConfig))
    {
        /* 读取配置文件失败，直接退出程序 */
        SPDLOG_ERROR("读取配置文件失败! 退出程序，{}", strBaseConfig.toStdString());
        exit(-1);
    }
    printBaseConfig();

    /* 读取算法配置 */
    QString strAlgorithmConfig = QCoreApplication::applicationDirPath() + "/config/Actions.ini";
    if(!GConfig.readAlgorithmConfig(strAlgorithmConfig))
    {
        /* 读取配置文件失败，直接退出程序 */
        SPDLOG_ERROR("读取配置文件失败! 退出程序，{}", strAlgorithmConfig.toStdString());
        exit(-1);
    }
    printAlgorithmConfig();

    /* 读取应用配置文件 */
    QString strAppConfig = QCoreApplication::applicationDirPath() + "/config/AppConfig.ini";
    if(!GConfig.readAppConfig(strAppConfig))
    {
        /* 读取配置文件失败，直接退出程序 */
        SPDLOG_ERROR("读取配置文件失败! 退出程序，{}", strAppConfig.toStdString());
        exit(-1);
    }
    printAppConfig();

    SPDLOG_INFO("========================================================");
}

/* 读取基础配置文件 */
bool GlobalConfig::readBaseConfig(const QString& strConfigFile)
{
    if(strConfigFile.isEmpty())
    {
        SPDLOG_ERROR("读取配置文件失败! 配置文件名为空");
        return false;
    }
    SPDLOG_DEBUG("读取配置文件: {}", strConfigFile.toStdString());
    QSettings settings(strConfigFile, QSettings::IniFormat);
    settings.setIniCodec("UTF-8");

    /* 读取WebAPI配置 */
    settings.beginGroup("WebAPI");
    m_webAPIUrl = settings.value("ServerURL", "").toString();
    m_webAPIKey = settings.value("ServerKey", "").toString();
    settings.endGroup();

    /* 读取Redis配置 */
    settings.beginGroup("Redis");
    m_redisIP = settings.value("URL", "").toString().toStdString();
    m_redisPort = settings.value("Port", 0).toInt();
    m_redisPWD = settings.value("Password", "").toString().toStdString();
    settings.endGroup();

    /* 读取SuperBrain配置 */
    settings.beginGroup("SuperBrain");
    m_superBrainUrl = settings.value("URL", "").toString().toStdString();
    m_superBrainAppKey = settings.value("AppKey", "").toString().toStdString();
    m_superBrainAppSecret = settings.value("AppSecret", "").toString().toStdString();
    settings.endGroup();

    return true;
}

/* 打印基础配置文件 */
void GlobalConfig::printBaseConfig()
{
    SPDLOG_INFO("========================================================");
    SPDLOG_INFO("WebAPI URL:            {}", m_webAPIUrl.toStdString());
    SPDLOG_INFO("WebAPI Key:            {}", m_webAPIKey.toStdString());
    SPDLOG_INFO("--------------------------------------------------------");
    SPDLOG_INFO("Redis IP:              {}", m_redisIP);
    SPDLOG_DEBUG("Redis Port:           {}", m_redisPort);
    SPDLOG_DEBUG("Redis Password:       {}", m_redisPWD);
    SPDLOG_INFO("--------------------------------------------------------");
    SPDLOG_INFO("SuperBrain URL:        {}", m_superBrainUrl);
    SPDLOG_DEBUG("SuperBrain AppKey:    {}", m_superBrainAppKey);
    SPDLOG_DEBUG("SuperBrain AppSecret: {}", m_superBrainAppSecret);
}


/* 读取算法配置文件 */
bool GlobalConfig::readAlgorithmConfig(const QString& strConfigFile)
{
    if(strConfigFile.isEmpty())
    {
        SPDLOG_ERROR("读取配置文件失败! 配置文件名为空");
        return false;
    }
    // SPDLOG_DEBUG("读取配置文件: {}", strConfigFile.toStdString());
    QSettings settings(strConfigFile, QSettings::IniFormat);
    settings.setIniCodec("UTF-8");

    settings.beginGroup("Actions");
    GVariable.ActFaceIdentify = settings.value("FaceIdentify", "").toString().toStdString();      /* 人脸识别算法ID */
    GVariable.ActPersonCount = settings.value("PersonCount", "").toString().toStdString();        /* 人员计数算法ID */
    GVariable.ActPlayPhone = settings.value("PlayPhone", "").toString().toStdString();            /* 玩手机算法ID */
    GVariable.ActContraband = settings.value("Contraband", "").toString().toStdString();          /* 违禁品算法ID */
    settings.endGroup();

    GVariable.m_mapAction.insert(std::make_pair(GVariable.ActFaceIdentify, "人脸识别"));
    GVariable.m_mapAction.insert(std::make_pair(GVariable.ActPersonCount, "人员计数"));
    GVariable.m_mapAction.insert(std::make_pair(GVariable.ActPlayPhone, "玩手机识别"));
    GVariable.m_mapAction.insert(std::make_pair(GVariable.ActContraband, "违禁品检测"));

    return true;
}

/* 打印算法配置文件 */
void GlobalConfig::printAlgorithmConfig()
{
    SPDLOG_DEBUG("========================================================");
    SPDLOG_DEBUG("人脸识别算法ID:       {}", GVariable.ActFaceIdentify);
    SPDLOG_DEBUG("人员计数算法ID:       {}", GVariable.ActPersonCount);
    SPDLOG_DEBUG("玩手机算法ID:         {}", GVariable.ActPlayPhone);
    SPDLOG_DEBUG("违禁品算法ID:         {}", GVariable.ActContraband);
}


/* 读取配置文件 */
bool GlobalConfig::readAppConfig(const QString& strConfigFile)
{
    if(strConfigFile.isEmpty())
    {
        SPDLOG_ERROR("读取配置文件失败! 配置文件名为空");
        return false;
    }
    // SPDLOG_DEBUG("读取配置文件: {}", strConfigFile.toStdString());
    QSettings settings(strConfigFile, QSettings::IniFormat);
    settings.setIniCodec("UTF-8");

    settings.beginGroup("System");

    GVariable.AppUpdateOnWorkTimeInterval_Time = settings.value("WorkOnInfoSecond", 600).toInt();  /* 更新在岗信息的时间间隔 */
    GVariable.AppPeopleOnWork_Time = settings.value("APPPEPOLEONWORK", 300).toInt();         /* 离岗时间 */
    GVariable.AppContraband_Time = settings.value("APPBADTHING", 50).toInt();                 /* 违禁物品出现的时间 */
    GVariable.AppBadMan_Time = settings.value("APPBADMAN", 50).toInt();                     /* 非法入侵 */
    GVariable.AppTired_Time = settings.value("APPTIRED", 50).toInt();                       /* 疲劳检测时间 */
    GVariable.AppPeopleCont = settings.value("APPPEPOLECONT", 50).toInt();             /* 人员聚集时间 */
    GVariable.AppPlayPhone = settings.value("APPPLAYPHONE", 50).toInt();               /* 玩手机识别 */
    GVariable.AppMouse = settings.value("APPMOUSE", 50).toInt();                       /* 手势识别 */
    GVariable.AppMask = settings.value("APPMASK", 5).toInt();                         /* 戴口罩识别 */

    GVariable.CheckSet = settings.value("CHECKSET", 300).toInt();                       /* 服务端多久检测一次配置 */
    GVariable.EventTimeValid = settings.value("EventTimeValid", 300).toInt();           /* 事件时间有效期 */

    settings.endGroup();

    return true;
}


/* 打印读取到的值 */
void GlobalConfig::printAppConfig()
{
    SPDLOG_INFO("========================================================");
    SPDLOG_INFO("APPPEPOLEONWORK:   {}", GVariable.AppPeopleOnWork_Time);
    SPDLOG_INFO("APPBADTHING:       {}", GVariable.AppContraband_Time);
    SPDLOG_INFO("APPBADMAN:         {}", GVariable.AppBadMan_Time);
    SPDLOG_INFO("APPTIRED:          {}", GVariable.AppTired_Time);
    SPDLOG_INFO("APPPEOPLECONT:     {}", GVariable.AppPeopleCont);
    SPDLOG_INFO("APPPLAYPHONE:      {}", GVariable.AppPlayPhone);
    SPDLOG_INFO("APPMOUSE:          {}", GVariable.AppMouse);
    SPDLOG_INFO("APPMASK:           {}", GVariable.AppMask);
    SPDLOG_INFO("CHECKSET:          {}", GVariable.CheckSet);
    SPDLOG_INFO("EventTimeValid:    {}", GVariable.EventTimeValid);
}

/* 添加通道信息 */
void GlobalConfig::setChannelInfo(std::map<int, std::string> mapChannelName)
{
    m_rwLockChnInfo.lockForWrite();
    m_mapChannelName.clear();
    m_mapChannelName = mapChannelName;
    m_rwLockChnInfo.unlock();
}

/* 清空通道信息 */
void GlobalConfig::clearChannelInfo()
{
    m_rwLockChnInfo.lockForWrite();
    m_mapChannelName.clear();
    m_rwLockChnInfo.unlock();
}

/* 获取通道名称 */
std::string GlobalConfig::getChannelName(int ChannelID)
{
    m_rwLockChnInfo.lockForRead();
    std::string strName = "";
    auto it = m_mapChannelName.find(ChannelID);
    if(it != m_mapChannelName.end())
    {
        strName = it->second;
    }
    m_rwLockChnInfo.unlock();
    return strName;
}

/* 添加摄像机信息 */
void GlobalConfig::setCameraInfo(std::map<int, std::string> mapCameraName)
{
    m_rwLockCamInfo.lockForWrite();
    m_mapCameraName.clear();
    m_mapCameraName = mapCameraName;
    m_rwLockCamInfo.unlock();
}
/* 获取摄像机名称 */
std::string GlobalConfig::getCameraName(int CameraID)
{
    m_rwLockCamInfo.lockForRead();
    std::string strName = "";
    auto it = m_mapCameraName.find(CameraID);
    if(it != m_mapCameraName.end())
    {
        strName = it->second;
    }
    m_rwLockCamInfo.unlock();
    return strName;
}



